part of hud;


/// Menus hold MenuObjects. They can be added to, generated, hidden, and unhidden
/// once given a parent element via a selector, such as '#my-menu'.
class Menu {

    //
    // STATIC

    static HashMap _menus = new HashMap<String, Menu>(); // <String, Menu>

    /// Get a menu by the ID it was registered with
    static Menu get(String id) {
      return _menus[id];
    }
    
    static Menu getFrom(Element e) {
      for (Menu m in _menus.values) {
        if (e.id == querySelector(m._selector))
          return m;
        for (MenuObject mo in m._menuObjects)
          if (mo.getElement() != null && mo.getElement().id == e.id)
            return m;
      }
      return null;
    }
    
    /// Discards a menu that has been registered
    static forget(String id) {
      _menus.remove(id);
    }
    
    
    //
    // INSTANCE
    
    // List of all the menu objects added via addMenuObject
    List<MenuObject> _menuObjects = new List<MenuObject>(); 
    
    // The HTML selector of the 'parent' Element, set by
    // the constructor
    String _selector;
    
    /// Makes a new menu given an id and HTML selector and stores it.
    /// The 'id' is the string it can be fetched with via Menu's staic 'get'
    /// and the selector is the parent element, such as '#my-menu'.
    Menu(String id, String selector) {
      _selector = selector;
      _menus[id] = this;
    }
    
    /// Adds a MenuObject to the menu to be built.
    Menu addMenuObject(MenuObject o) {
      _menuObjects.add(o);
      o._associate(_selector);
      return this;
    }
    
    /// Renders all the MenuObjects in the menu to the
    /// parent element supplied by the associated selector.
    build() {
        querySelector(_selector).innerHtml = '';
        for (MenuObject m in _menuObjects) m.buildElement();
    }
    
    // The 'display' css attribute is remembers when hide() is used
    // so that it can be applied on show()
    String _display;
    
    /// Hides the menu by setting its parent object's css display to 'none'
    hide() {
      Element menu = querySelector(_selector);
      _display = menu.style.display;
      menu.style.display = 'none';
    }
    
    /// Shows the parent element by altering its css display.
    /// Has optional display argument to use, otherwise
    /// it uses the same display that was present when hide()
    /// was last used (or failing that, 'block').
    show({display: String}) {
      if (_display != null && _display != 'none')
        querySelector(_selector).style.display = _display;
      else
        querySelector(_selector).style.display = (display != null ? display : 'block');
    }
}


class MenuHandler {
  
  //
  // HANDLERS

  /// Initialized internally. No need to call manually.
  static void initialize() {
      // Menu items do stuff on click
      querySelector('body')
          ..onClick.listen((Event e) {
        // Check the element to see if it's a 'menuobject'
        Element el = e.target;
        if (!(el is DivElement))
            el = findClosestAncestor('div', el);
        
        if (!el.classes.contains('mo'))
                  return;
                
        // Get the MenuObject from the element
        MenuObject.getFrom(el).click();
      });
  }

}



/// Objects intended to be 'MenuObjects' must implement this class.
abstract class MenuObject {
  
  String _id;
  
  String _parent;
  
  /// Get the MenuObject instance of a matching Element.
  static getFrom(Element e) {
    // Iterating is probably the easiest way.
    for (Menu m in Menu._menus.values) {
      for (MenuObject mo in m._menuObjects)
        if (mo.getElement() != null && mo.getElement().id == e.id)
          return mo;
    }
    return null;
  }
  
  Element getElement();
    
  buildElement();
  
  click();
  
  _associate(String menu_selector);
  
}



/// MainMenuObjects fetch and render different pages when clicked.
/// They also control the 'page icon' if marked 'active'.
class MainMenuObject implements MenuObject {

    //
    // INSTANCED

    String _id, _tip, _icon, _parent;

    // Active menu items have different css and control
    // the 'page icon'
    bool _active;

    /// Construct a new MainMenuObject
    MainMenuObject(String icon, String page, String tooltip) {
        _id = page;
        _tip = tooltip;
        _icon = icon;
        _active = false;
    }
    
    _associate(String _selector) {
        _parent = _selector;  
    }

    /// Gets the HTML DivElement associated with the menu item
    Element getElement() {
        return querySelector('#mo_$_id');
    }

    /// Adds the necessary information to the DOM
    buildElement() {
        // Escape the tooltip.
        String tooltip = C.HTML_ESCAPE.convert(_tip);

        Element mb = querySelector(_parent);

        if (_active) {
            // Insert HTML into the menu-bar
            mb.insertAdjacentHtml('beforeEnd', ''' 
          <div id='mo_$_id' class='mo menu-item-active'>
            <img src='../../packages/shuttlehud/icons/$_icon.svg'>
          </div>
        ''');

        } else {

            // Insert HTML into the menu-bar
         mb.insertAdjacentHtml('beforeEnd', ''' 
          <div id='mo_$_id' class='mo menu-item tooltip' tooltip='$tooltip'>
            <img src='../../packages/shuttlehud/icons/$_icon.svg'>
          </div>
        ''');

        }
    }

    setActive(bool a) {
        _active = a;
    }
    
    click() {
      print('Click! ' + this.toString());
    }

}



class PopupButton implements MenuObject {

  static Set<Element> registered = new HashSet<Element>();

  //
  // HANDLERS

  // Initialize listeners and such
  static void initialize() {
    
  }

  static _mutation(List<MutationRecord> mutations, MutationObserver observer) {
    for (Element e in querySelectorAll('.popupButton')) {
      if (e is DivElement && !registered.contains(e)) {
        e ..onMouseEnter.listen(_enter)
          ..onMouseLeave.listen(_leave)
          ..onMouseDown.listen(_down)
          ..onMouseUp.listen(_up);
        registered.add(e);
      }
    }
  }
  
  static void _enter(Event e) {
    // Grab the target
    Element el = e.target;
     el..classes.add('popupButton_hover');

  }

  static void _down(Event e) {
      Element el = e.target;
      if (!(el is DivElement))
          el = findClosestAncestor('div', el);
      el
          ..classes.add('popupButton_click')
          ..classes.remove('popupButton_hover');
  }

  static void _up(Event e) {
      Element el = e.target;
      if (!(el is DivElement))
          el = findClosestAncestor('div', el);
      el ..classes.remove('popupButton_click')
         ..classes.add('popupButton_hover');
  }

  static void _leave(Event e) {
    Element el = e.target;
    el ..classes.remove('popupButton_click')
       ..classes.remove('popupButton_hover');
  }
  
  
  
  //
  // INSTANCED

  String _id, _tip, _icon, _parent;
  
  HashMap<String, Object> vars = new HashMap<String, Object>();
  
  Function click_function = null;
  
  _associate(String _selector) {
    _parent = _selector;
  }
  
  /// Construct a new MainMenuObject
  PopupButton(String icon, String tooltip, Function on_click) {
      _id = new Uuid().v4();
      click_function = on_click;
      _tip = tooltip;
      _icon = icon;
  }

  /// Gets the HTML DivElement associated with the menu item
  Element getElement() {
      return querySelector('#mo_$_id');
  }

  /// Adds the necessary information to the DOM
  buildElement() {
      // Escape the tooltip.
      String tooltip = C.HTML_ESCAPE.convert(_tip);

      Element mb = querySelector(_parent);

      // Insert HTML into the menu-bar
      mb.insertAdjacentHtml('beforeEnd', ''' 
        <div id='mo_$_id' class='mo popupButton tooltip' tooltip='$tooltip'>
          <img src='../../packages/shuttlehud/icons/$_icon.svg'>
        </div>
        ''');

      }

  click() {
    click_function(this);
  }

  
}
